from __future__ import annotations

from typing import Any


class Signal:
    """
    Handles datatransportation between modules in the processor.
    Each signal has one source module and can have multiple
    destination modules.
    """
    def __init__(self, processor: Processor, name: str = "Signal", value = None):
        self.processor = processor
        self.name = name
        self.source = None
        self.destinations = set()
        self.value = value

    def add_destination(self, module: Module) -> None:
        """
        Add a destination module that the signal will send new
        values to.
        """
        self.destinations.add(module)

    def set_source(self, module: Module) -> None:
        """
        Set source module that will send values through the signal.
        """
        self.source = module

    def update_value(self, value) -> None:
        """
        Sets value in signal and queues destination moduels for update.
        """
        if value != self.value:
            self.value = value
            # Inform the processor on what modules have been given
            # signal so they can update
            for destination in self.destinations:
                self.processor.add_modules_to_update(destination)

    def get_value(self) -> Any:
        """
        Get the signals current value.
        """
        return self.value

    def set_value(self, value) -> None:
        """
        Set the value of the signal without queuing the destination
        modules of the signal for update. This method should not
        be called by a module.
        """
        self.value = value

    def get_name(self) -> str:
        return self.name
