from typing import Optional

from qtpy.QtCore import Qt
from qtpy.QtCore import Signal as pyqtSignal
from qtpy.QtWidgets import (
    QComboBox,
    QDialog,
    QHBoxLayout,
    QLineEdit,
    QPushButton,
    QWidget,
)

from simudator.core import Module


class MemoryContentDialog(QDialog):
    accepted = pyqtSignal(str, str, str, name="accepted")

    def __init__(
        self,
        memory: Module,
        parent: Optional['QWidget'] = None,
        flags: Qt.WindowFlags | Qt.WindowType = Qt.WindowFlags(),
    ) -> None:
        super().__init__(parent, flags)

        self.memory = memory
        self.memory_name = memory.get_state()['name']

        adresses = [str(i) for i in range(len(memory.get_state()['memory']))]

        self.adressSelectWidget = QComboBox()
        self.adressSelectWidget.addItems(adresses)
        self.adressSelectWidget.activated.connect(self.updateValue)
        self.adressSelectWidget.setEditable(True)

        self.valuesWidget = QLineEdit()

        okButton = QPushButton('OK')
        okButton.clicked.connect(self.signalAccepted)
        cancelButton = QPushButton('Cancel')
        cancelButton.clicked.connect(self.close)

        hboxLayout = QHBoxLayout()
        hboxLayout.addWidget(self.adressSelectWidget, 0)
        hboxLayout.addWidget(self.valuesWidget, 1)
        hboxLayout.addWidget(okButton)
        hboxLayout.addWidget(cancelButton)
        self.setLayout(hboxLayout)

        self.show()
        self.updateValue()

    def updateValue(self) -> None:
        selectedAdress = int(self.adressSelectWidget.currentText(), 16)
        memory_content = self.memory.get_state()['memory']
        value = memory_content[selectedAdress]
        self.valuesWidget.setText(str(value))

    def signalAccepted(self) -> None:
        adress = self.adressSelectWidget.currentText()
        enteredValues = self.valuesWidget.text()
        self.accepted.emit(self.memory_name, adress, enteredValues)
        self.close()
