from simudator.core.modules.register import IntegerRegister
from simudator.core.signal import Signal
from simudator.processor.mia.modules.mia_bus_connect import MiaBusConnector


class AR(IntegerRegister, MiaBusConnector):
    """
    Register for saving AlU calculations in MIA.
    """

    def __init__(
        self,
        alu_input_signal: Signal,
        alu_output_signal: Signal,
        bus_output_signal: Signal,
        bus_control_signal: Signal,
        bus_id: int,
        bit_length: int,
        name="AR",
    ) -> None:
        self.alu_output_signal = alu_output_signal

        IntegerRegister.__init__(
            self, alu_input_signal, bus_output_signal, bit_length, name=name
        )
        MiaBusConnector.__init__(self, bus_control_signal, bus_id)

    def output_register(self) -> None:
        # always update alu
        self.alu_output_signal.update_value(self.value)

        # only update bus when asked too
        if self.write_to_bus():
            IntegerRegister.output_register(self)
        else:
            self.output_s.update_value(None)

    def update_logic(self) -> None:
        if self.write_to_bus():
            IntegerRegister.output_register(self)
        else:
            self.output_s.update_value(None)

    def get_input_signals(self) -> list[Signal]:
        return [self.input_s]

    def get_output_signals(self) -> list[Signal]:
        return [self.alu_output_signal, self.output_s, self.bus_control_s]

    def print_module(self):
        print(
            "",
            self.name,
            "\n -----",
            "\n value: ",
            hex(self.value),
            "\n bit length: ",
            self.bit_length,
        )
