from simudator.core.modules.register import IntegerRegister
from simudator.core.signal import Signal
from simudator.processor.mia.modules.mia_bus_connect import MiaBusConnector


class ASR(IntegerRegister, MiaBusConnector):
    """
    Register for controlling the memory andress in MIA.
    """

    def __init__(
        self,
        bus_input_signal: Signal,
        memory_output_signal: Signal,
        bus_control_signal: Signal,
        bus_id: int,
        bit_length: int,
        name="ASR",
    ) -> None:
        IntegerRegister.__init__(
            self, bus_input_signal, memory_output_signal, bit_length, name=name
        )
        MiaBusConnector.__init__(self, bus_control_signal, bus_id)

    def update_register(self) -> None:
        if self.read_from_bus():
            IntegerRegister.update_register(self)

    def update_logic(self) -> None:
        if self.write_to_bus():
            self.output_register()

    def get_input_signals(self) -> [Signal]:
        return [self.input_s, self.bus_control_s]

    def get_output_signals(self) -> [Signal]:
        return [self.output_s]

    def print_module(self):
        print(
            "",
            self.name,
            "\n -----",
            "\n value: ",
            hex(self.value),
            "\n bit length: ",
            self.bit_length,
        )
