from simudator.core.modules.register import IntegerRegister
from simudator.core.signal import Signal
from simudator.processor.mia.modules.mia_bus_connect import MiaBusConnector


class HR(IntegerRegister, MiaBusConnector):
    """
    Register for saving large AlU calculations in MIA.
    """

    def __init__(
        self,
        alu_input_signal: Signal,
        alu_output_signal: Signal,
        bus_input_signal: Signal,
        bus_output_signal: Signal,
        bus_control_signal: Signal,
        bus_id: int,
        bit_lenght: int,
        name="HR",
    ) -> None:
        self.alu_output_signal = alu_output_signal
        self.alu_input_signal = alu_input_signal

        self.alu_input_signal.add_destination(self)

        IntegerRegister.__init__(
            self, bus_input_signal, bus_output_signal, bit_lenght, name=name
        )
        MiaBusConnector.__init__(self, bus_control_signal, bus_id)

    def update_register(self) -> None:
        """
        Updates the internal values from the signals from the bus and
        the alu. It only updates the bus when asked to, and will always
        update the alu signal.
        """

        if self.read_from_bus():
            IntegerRegister.update_register(self)

        elif self.alu_input_signal.get_value() is not None:
            input_value = self.alu_input_signal.get_value()
            self.value = input_value & self.mask

    def output_register(self) -> None:
        """
        Outputs the internal values to the signals to the bus and
        the alu. It only outputs the bus when asked to, and will always
        output the alu signal.
        """

        self.alu_output_signal.update_value(self.value)

        if self.write_to_bus():
            IntegerRegister.output_register(self)

        else:
            self.output_s.update_value(None)

    def update_logic(self):
        self.output_register()

    def print_module(self):
        print(
            "",
            self.name,
            "\n -----",
            "\n value: ",
            hex(self.value),
            "\n bit length: ",
            self.bit_length,
        )

    def get_input_signals(self) -> list[Signal]:
        return [self.alu_input_signal, self.input_s]

    def get_output_signals(self) -> list[Signal]:
        return [
            self.alu_output_signal,
            self.bus_control_s,
            self.output_s,
        ]
