import sys

from qtpy.QtWidgets import QApplication

from simudator.cli.cli import CLI
from simudator.core.modules import Memory, Register
from simudator.core.processor import Processor, Signal
from simudator.gui.gui import GUI
from simudator.gui.module_graphics_item.memory_graphic import MemoryGraphicsItem
from simudator.gui.module_graphics_item.register_graphic import RegisterGraphicsItem


class SIMPLE_CPU(Processor):
    """
    This cpu is made to test core modules and module graphics items for these modules.
    """

    def __init__(self) -> None:
        super().__init__()
        # Creating all signals
        # Signals follow the naming convention 'to_from'
        self.max_line_len = 100

        # Dummy values used when the values in the constructor is not needed
        # unused = Signal(self)

        # Needed since optional arguments need to be in the correct order

        mem_input = Signal(self, "mem_input")
        mem_output = Signal(self, "mem_output")
        mem_control = Signal(self, "mem_control")
        mem_adress = Signal(self, "mem_adress")
        empty_input = Signal(self, "empty_input", value=None)
        empty_output = Signal(self, "empty_output", value=None)

        self.memory = Memory(mem_input, mem_output, mem_control, mem_adress, 10)
        self.input_register = Register(empty_input, mem_input, name="Input Reg")
        self.output_register = Register(mem_output, empty_output, name="Output Reg")
        self.control_register = Register(empty_input, mem_control, name="Control Reg")
        self.adress_register = Register(empty_input, mem_adress, name="Adress Reg")

        ALLTHEMODULES = [
            self.memory,
            self.input_register,
            self.output_register,
            self.control_register,
            self.adress_register,
        ]

        for module in ALLTHEMODULES:
            self.add_module(module)

        self.lambdas = {}

    def launch_gui(self):
        app = QApplication(sys.argv)
        self.reset()
        gui = GUI(self)

        gui.addModuleGraphicsItem(MemoryGraphicsItem(self.memory))
        gui.addModuleGraphicsItem(RegisterGraphicsItem(self.input_register))
        gui.addModuleGraphicsItem(RegisterGraphicsItem(self.output_register))
        gui.addModuleGraphicsItem(RegisterGraphicsItem(self.control_register))
        gui.addModuleGraphicsItem(RegisterGraphicsItem(self.adress_register))

        gui.addAllSignals()
        gui.show()
        app.exec()

    def launch_cli(self):
        cli = CLI(self)
        self.reset()
        cli.run()


if __name__ == "__main__":
    # this is bad terminal code, but who cares
    cpu = SIMPLE_CPU()
    flag = None
    try:
        flag = sys.argv[1]
    except IndexError:
        pass

    if flag in ("gui", "g", "-gui"):
        cpu.launch_gui()
    else:
        cpu.launch_cli()
