from simudator.core.modules.register import Register
from simudator.core.processor import Processor
from simudator.core.signal import Signal


def test_default_name():
    cpu = Processor()
    s = Signal(cpu)
    register = Register(s, s, 8)
    assert register.name == "Register"


def test_name():
    cpu = Processor()
    s = Signal(cpu)
    register = Register(s, s, 13, "reg")
    assert register.name == "reg"


def test_set_state():
    cpu = Processor()
    s = Signal(cpu)
    register = Register(s, s, 4)

    state = {"name": "Reg", "value": 14}
    register.set_state(state)

    assert register.name == "Reg"
    assert register.value == 14


def test_get_state():
    cpu = Processor()
    s = Signal(cpu)
    register = Register(s, s, 13, "register")
    state = register.get_state()

    assert state["name"] == "register"
    assert state["value"] == 13


def test_signal_propagation():
    # Modules needed to run the test
    cpu = Processor()
    input_s = Signal(cpu)
    output_s = Signal(cpu)
    register = Register(input_s, output_s, 13)

    cpu.add_module(register)

    input_s.update_value(10)
    cpu.do_tick()

    assert register.value == 10
